class Planetarium
{
   public static double kepler( double m, double exz, double eps )
   {
      double oldE = 0;
      double newE = m;
      while( Math.abs( newE - oldE ) > eps )
      {
         // Fixpunkt-Iteration: E = M + exz sin(E)
         oldE = newE;
         newE = m + exz * Math.sin( oldE );
      }
      return 2. * Math.atan( Math.tan( newE / 2. ) * Math.sqrt( ( 1. + exz )/( 1. - exz ) ) );
   }

   public static double rphi( double phi, double exz, double a )
   {
      // min: r(peri) = a ( 1 - exz )  perigaeum, perihel
      // max: r(apo)  = a ( 1 + exz )   apogaeum,  apohel
      // grosse Halbachse: a = ( r(peri) + r(apo) ) / 2
      // kleine Halbachse: b = a sqrt( 1 - exz^2 ) = sqrt( r(peri) r(apo) )
      
      return a * ( 1. - exz * exz ) / ( 1. + exz * Math.cos( phi ) );
   }

   public static double [] kepler( double [] m, double exz, double eps )
   {
      int n = m.length;
      double [] result = new double[n];
      for( int i = 0; i < n; i++ )
      {
         result[i] = kepler( m[i], exz, eps );
      }
      return result;
   }

   public static double [] rphi( double [] phi, double exz, double a )
   {
      int n = phi.length;
      double [] result = new double[n];
      for( int i = 0; i < n; i++ )
      {
         result[i] = rphi( phi[i], exz, a );
      }
      return result;
   }

   public static JxnRealArrayAlgebra kepler( JxnRealArrayAlgebra m, double exz, double eps )
   {
      return new JxnRealArrayAlgebra( kepler( m.getValue(), exz, eps ) );
   }

   public static JxnRealArrayAlgebra rphi( JxnRealArrayAlgebra phi, double exz, double a )
   {
      return new JxnRealArrayAlgebra( rphi( phi.getValue(), exz, a ) );
   }
}
