import org.apache.commons.math3.analysis.MultivariateFunction;

/** Wrapper used to pass a method definition to <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms.
  * Computes a command given as a String.<br/>
  * <br/>
  * See <a target="_top" href="../0_CommonsMath_README.html#examples">JxnMultivariateFunction_Optimization~Demo.jxn</a>
  * for an example.  
  */
public class JxnMultivariateFunction implements MultivariateFunction
{
   String cmd;
   KmgFormelInterpreter fi;
   static final boolean skipInputLogEnabled = true;

   /** Constructs a {@link org.apache.commons.math3.analysis.MultivariateFunction}
     * instance to be used by <a target="_blank" href="http://commons.apache.org/math">Commons Math</a> algorithms.<br/>
     * Example:
     * <pre>
     *    mvf = @JxnMultivariateFunction( $this, "( $x[0]^2 + 2.5 $x[1]^2 - $x[1] ) * Math.exp( 1 - $x[0]^2 - $x[1]^2 );" )</pre>
     * @param fi an internal copy of {@code fi} is used to execute {@code cmd}
     * @param cmd one or more statements separated by '{@code ;}'. The command {@code cmd} must use the
     * array {@code double[] $x}
     * to compute a result of type double. If {@code cmd} ends with '{@code ;}', log output of intermediate steps is suppressed.
     */
   public JxnMultivariateFunction( KmgFormelInterpreter fi, String cmd )
   {
      this.cmd = cmd;
      this.fi = new KmgFormelInterpreter( fi );
   }
   
   /** @deprecated replaced by {@link #JxnMultivariateFunction(KmgFormelInterpreter, String)} */
   public JxnMultivariateFunction( String cmd, KmgFormelInterpreter fi )
   {
      this.cmd = cmd;
      this.fi = new KmgFormelInterpreter( fi );
   }
   
   /** Internally called by Commons Math algorithms: Computes the function value using the given argument {@code x}. */
   public double value( double [] x )
   {
      fi.put( "$x", x );
      return KmgFormelInterpreter.doubleValue( fi.execute( cmd, skipInputLogEnabled ) );
   }

   /** Returns the command {@code cmd} passed to the constructor. */
   public String toString()
   {
      return cmd;
   }
}
